#!/usr/bin/env python
# -*- coding: utf-8 -*-
##
##    Copyright (C) 2009 manatlan manatlan[at]gmail(dot)com
##
## This program is free software; you can redistribute it and/or modify
## it under the terms of the GNU General Public License as published
## by the Free Software Foundation; version 2 only.
##
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
##
## More infos : http://www.manatlan.com/page/totem-plugin-flux
##

from libs.path import getHomePath
from libs.m3u import M3U
from libs.web import getFilePage
from libs.flux import xml2vids
from libs.rss import mkRssFile

import os,re,urllib,hashlib
CACHE=60*55 #minutes

RSS=[
    ("SAV","http://www.rss-one.com/rss-6690-Sav-des-Emissions"),
    ("Zapping","http://www.rss-one.com/rss-3950-Canal-Plus---Le-Zapping"),
    ("Koreus","http://feeds.feedburner.com/Koreus-podcasts-video"),

    ("Jonathan Lambert", "http://gdata.youtube.com/feeds/base/users/jonathanlambertfr/uploads?alt\x3drss\x26v\x3d2\x26orderby\x3dpublished\x26client\x3dytapi-youtube-profile"),
    ("Nimportequi.com",  "http://gdata.youtube.com/feeds/base/users/nqtv/uploads?alt\x3drss\x26v\x3d2\x26orderby\x3dpublished\x26client\x3dytapi-youtube-profile"),
    ("FranceInter Humour", "http://www.dailymotion.com/rss/playlist/xo10o_franceinter_humour"),
    #~ ("Leweb2zero", "http://leweb2zero.tv/rss"),
    ("Trash tv", "http://feeds.feedburner.com/explicit-tv"),
    ("Dailymotion fun", "http://www.dailymotion.com/rss/fr/featured/channel/fun"),
    ("Dailymotion selection humour", "http://www.dailymotion.com/rss/fr/quality-watcher/channel/fun"),
    ("Les graves JT", "http://www.dailymotion.com/rss/playlist/xy0m1_Les_Graves_Infos_les-graves-jt"),

    ]


def getDailymotionFlvs(url):
    """ return a dict of dailymotion videos """
    file=getFilePage(url,cache=CACHE)
    buf = open(file).read()
    g=re.search('addVariable\("video", "([^"]+)"\);',buf)
    d={}
    if g:
        list=urllib.unquote(g.groups(0)[0]).split("||")
        for i in list:
            url,nom=i.split("@@")
            d[nom]="http://www.dailymotion.com"+url
    return d


def resolve_t_param (u):
    """We have to get the t parameter from the actual video page, since Google changed how their URLs work"""
    f=getFilePage(u,cache=CACHE)
    stream = open(f)
    regexp1 = re.compile ("swfArgs.*\"t\": \"([^\"]+)\"")
    regexp2 = re.compile ("</head>")

    contents = stream.read ()
    if contents != "":
        """Check for the t parameter, which is now in a JavaScript array on the video page"""
        matches = regexp1.search (contents)
        if (matches != None):
            stream.close ()
            return matches.group (1)

        """Check to see if we've come to the end of the <head> tag; in which case, we should give up"""
        if (regexp2.search (contents) != None):
            stream.close ()
            return ""

    stream.close ()
    return ""

def getRealYoutubeUrl(url):
    p=resolve_t_param(url)
    youtube_id = url.split("=")[-1]
    if p:
        return "http://www.youtube.com/get_video?video_id=" + urllib.quote (youtube_id) + "&t=" + urllib.quote (p) + "&fmt=18"

def getRealDailymotionUrl(url):
    d=getDailymotionFlvs(url)
    if d:
        return d["spark"]

def isVideoDailymotion(u):
    return ("dailymotion" in u) and ("/video/" in u)

def isVideoYoutube(u):
    return ("youtube.com" in u) and ("watch" in u)

def getVideoUrl(u):
    if isVideoDailymotion(u):
        return getRealDailymotionUrl(u)
    elif isVideoYoutube(u):
        return getRealYoutubeUrl(u)
    else:
        return u


def getPlayListrss(url):
    """return a METHOD which build the rss"""
    url=url
    def _get():
        buf=file(getFilePage(url,CACHE)).read()
        ll=[]
        l=xml2vids(buf)
        for idx,v in enumerate(l):
            nom,u=v
            u=getVideoUrl(u)
            if u:
                ll.append((nom,u ))
                yield float(idx+1) /len(l)
            else:
                print "video not available : ",u
        yield mkRssFile(hashlib.md5(url).hexdigest(),ll)
    return _get


class MyM3U(M3U):
    def __init__(self,l=[]):
        M3U.__init__(self,l)

    def get(self,idx):  # override original getter
        nom,url = M3U.__getitem__(self,idx)
        return (nom,getPlayListrss(url))


class Fetch:
    name = "Playlists"

    @staticmethod
    def get():
        file = os.path.join(getHomePath(),".totem_flux_playlists.m3u")
        if not os.path.isfile(file):
            p=MyM3U(RSS)
            p.save(file)
        else:
            p=MyM3U()
            p.load(file)

        return p


if __name__ == "__main__":
    uy = getRealYoutubeUrl("http://www.youtube.com/watch?v=0CC79bZPrEY")
    ud = getRealDailymotionUrl("http://www.dailymotion.com/video/x9ecmi_le-gang-des-doudous_fun?from=rss")

    assert getVideoUrl("kiki") == "kiki"
    assert getVideoUrl("http://www.youtube.com/watch?v=0CC79bZPrEY") == uy
    assert getVideoUrl("http://www.dailymotion.com/video/x9ecmi_le-gang-des-doudous_fun?from=rss") == ud


    ll=Fetch.get()
    for idx in range(0,len(ll)):
        n,m=ll.get(idx)
        print n,m()
